@@ -61,16 +61,17 @@ import javax.xml.validation.Validator
 import javax.xml.xpath.XPath
 import javax.xml.xpath.XPathConstants
 import javax.xml.xpath.XPathFactory
-import logger
+
+private val logger: Logger = LoggerFactory.getLogger("tech.libeufin.util.XMLUtil")
 
 class DefaultNamespaces : NamespacePrefixMapper() {
     override fun getPreferredPrefix(namespaceUri: String?, suggestion: String?, requirePrefix: Boolean): String? {
         if (namespaceUri == "http://www.w3.org/2000/09/xmldsig#") return "ds"
+        if (namespaceUri == XMLConstants.W3C_XML_SCHEMA_INSTANCE_NS_URI) return "xsi"
         return null
     }
 }
 
-
 class DOMInputImpl : LSInput {
     var fPublicId: String? = null
     var fSystemId: String? = null
@@ -227,11 +228,14 @@ class XMLUtil private constructor() {
             }
             val schemaInputs: Array<Source> = listOf(
                 "xsd/ebics_H004.xsd",
+                "xsd/ebics_H005.xsd",
                 "xsd/ebics_hev.xsd",
                 "xsd/camt.052.001.02.xsd",
                 "xsd/camt.053.001.02.xsd",
                 "xsd/camt.054.001.02.xsd",
-                "xsd/pain.001.001.03.xsd"
+                "xsd/pain.001.001.03.xsd",
+                // "xsd/pain.001.001.03.ch.02.xsd", // Swiss 2013 version.
+                "xsd/pain.001.001.09.ch.03.xsd" // Swiss 2019 version.
             ).map {
                 val stream =
                     classLoader.getResourceAsStream(it) ?: throw FileNotFoundException("Schema file $it not found.")
@@ -289,23 +293,35 @@ class XMLUtil private constructor() {
             return validate(xmlSource)
         }
 
-        inline fun <reified T> convertJaxbToString(obj: T): String {
+        inline fun <reified T> convertJaxbToString(
+            obj: T,
+            withSchemaLocation: String? = null
+            ): String {
             val sw = StringWriter()
             val jc = JAXBContext.newInstance(T::class.java)
             val m = jc.createMarshaller()
             m.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, true)
+            if (withSchemaLocation != null) {
+                m.setProperty(Marshaller.JAXB_SCHEMA_LOCATION, withSchemaLocation)
+            }
             m.setProperty("com.sun.xml.bind.namespacePrefixMapper", DefaultNamespaces())
             m.marshal(obj, sw)
             return sw.toString()
         }
 
-        inline fun <reified T> convertJaxbToDocument(obj: T): Document {
+        inline fun <reified T> convertJaxbToDocument(
+            obj: T,
+            withSchemaLocation: String? = null
+        ): Document {
             val dbf: DocumentBuilderFactory = DocumentBuilderFactory.newInstance()
             dbf.isNamespaceAware = true
             val doc = dbf.newDocumentBuilder().newDocument()
             val jc = JAXBContext.newInstance(T::class.java)
             val m = jc.createMarshaller()
             m.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, true)
+            if (withSchemaLocation != null) {
+                m.setProperty(Marshaller.JAXB_SCHEMA_LOCATION, withSchemaLocation)
+            }
             m.setProperty("com.sun.xml.bind.namespacePrefixMapper", DefaultNamespaces())
             m.marshal(obj, doc)
             return doc
@@ -337,8 +353,6 @@ class XMLUtil private constructor() {
             val tf = TransformerFactory.newInstance()
             val t = tf.newTransformer()
 
-            //t.setOutputProperty(OutputKeys.INDENT, "yes")
-
             /* Make string writer.  */
             val sw = StringWriter()
 
@@ -402,12 +416,16 @@ class XMLUtil private constructor() {
         /**
          * Sign an EBICS document with the authentication and identity signature.
          */
-        fun signEbicsDocument(doc: Document, signingPriv: PrivateKey): Unit {
+        fun signEbicsDocument(
+            doc: Document,
+            signingPriv: PrivateKey,
+            withEbics3: Boolean = false
+        ) {
             val xpath = XPathFactory.newInstance().newXPath()
             xpath.namespaceContext = object : NamespaceContext {
                 override fun getNamespaceURI(p0: String?): String {
                     return when (p0) {
-                        "ebics" -> "urn:org:ebics:H004"
+                        "ebics" -> if (withEbics3) "urn:org:ebics:H005" else "urn:org:ebics:H004"
                         else -> throw IllegalArgumentException()
                     }
                 }
@@ -450,12 +468,16 @@ class XMLUtil private constructor() {
             authSigNode.removeChild(innerSig)
         }
 
-        fun verifyEbicsDocument(doc: Document, signingPub: PublicKey): Boolean {
+        fun verifyEbicsDocument(
+            doc: Document,
+            signingPub: PublicKey,
+            withEbics3: Boolean = false
+        ): Boolean {
             val xpath = XPathFactory.newInstance().newXPath()
             xpath.namespaceContext = object : NamespaceContext {
                 override fun getNamespaceURI(p0: String?): String {
                     return when (p0) {
-                        "ebics" -> "urn:org:ebics:H004"
+                        "ebics" -> if (withEbics3) "urn:org:ebics:H005" else "urn:org:ebics:H004"
                         else -> throw IllegalArgumentException()
                     }
                 }
