@@ -5,7 +5,6 @@ import tech.libeufin.util.CryptoUtil
 import java.math.BigInteger
 import java.security.interfaces.RSAPublicKey
 import java.util.*
-import javax.swing.text.Segment
 import javax.xml.bind.annotation.*
 import javax.xml.bind.annotation.adapters.CollapsedStringAdapter
 import javax.xml.bind.annotation.adapters.HexBinaryAdapter
@@ -161,7 +160,7 @@ class EbicsRequest {
         lateinit var orderType: String
 
         /**
-         * Only present if this ebicsRequest is a upload order
+         * Only present if this ebicsRequest is an upload order
          * relating to an already existing order.
          */
         @get:XmlElement(name = "OrderID", required = true)
@@ -289,7 +288,7 @@ class EbicsRequest {
     companion object {
 
         fun createForDownloadReceiptPhase(
-            transactionId: String,
+            transactionId: String?,
             hostId: String
 
         ): EbicsRequest {
@@ -315,7 +314,6 @@ class EbicsRequest {
                     }
                 }
             }
-
         }
 
         fun createForDownloadInitializationPhase(
@@ -444,7 +442,7 @@ class EbicsRequest {
 
         fun createForUploadTransferPhase(
             hostId: String,
-            transactionId: String,
+            transactionId: String?,
             segNumber: BigInteger,
             encryptedData: String
         ): EbicsRequest {
@@ -477,7 +475,7 @@ class EbicsRequest {
 
         fun createForDownloadTransferPhase(
             hostID: String,
-            transactionID: String,
+            transactionID: String?,
             segmentNumber: Int,
             numSegments: Int
         ): EbicsRequest {
